// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package implments

import (
	dal "LAPP_GAAS_GFrame_BACKEND/dao/report"
	"LAPP_GAAS_GFrame_BACKEND/db"
	"LAPP_GAAS_GFrame_BACKEND/grmi"
	meta "LAPP_GAAS_GFrame_BACKEND/meta/report"
	model "LAPP_GAAS_GFrame_BACKEND/models/report"
	"LAPP_GAAS_GFrame_BACKEND/utils"
	"errors"
	"fmt"
	"reflect"
	"strconv"
	"time"
)

/******************************************************************************
*
* @Description              : FilterDataInfo的默认查询条件
*
* @Author                   : 代码生成器创建
*
* @Date                     : 2021-06-21 09:55:32
*
******************************************************************************/
var DefaultConditionOfFilterDataInfo = grmi.NewCondition(
	map[string]grmi.ConditionItem{
		meta.FilterDataInfo_Station.UrlParameterName:  grmi.NewConditionItem(meta.FilterDataInfo_Station, grmi.Equal, false),
		meta.FilterDataInfo_Ts.UrlParameterName:       grmi.NewConditionItem(meta.FilterDataInfo_Ts, grmi.Approximate, false),
		meta.FilterDataInfo_Address.UrlParameterName:  grmi.NewConditionItem(meta.FilterDataInfo_Address, grmi.Equal, false),
		meta.FilterDataInfo_Value.UrlParameterName:    grmi.NewConditionItem(meta.FilterDataInfo_Value, grmi.Equal, false),
		meta.FilterDataInfo_Duration.UrlParameterName: grmi.NewConditionItem(meta.FilterDataInfo_Duration, grmi.Equal, false),
		meta.FilterDataInfo_DescCN.UrlParameterName:   grmi.NewConditionItem(meta.FilterDataInfo_DescCN, grmi.Equal, false),
		meta.FilterDataInfo_PartNo.UrlParameterName:   grmi.NewConditionItem(meta.FilterDataInfo_PartNo, grmi.Equal, false),
	},
	nil,
)

/******************************************************************************
*
* @Description              : FilterDataInfo的默认分页查询条件
*
* @Author                   : 代码生成器创建
*
* @Date                     : 2021-06-21 09:55:32
*
******************************************************************************/
var DefaultConditionOfFilterDataInfoAndPaging = grmi.NewCondition(
	DefaultConditionOfFilterDataInfo.Items,
	grmi.NewPaging(),
)

/******************************************************************************
*
* @Struct Name              : FilterDataInfoServiceImplement
*-----------------------------------------------------------------------------
*
* @Description              : FilterDataInfo的服务实现
*
* @Author                   : 代码生成器创建
*
* @Date                     : 2021-06-21 09:55:32
*
******************************************************************************/
type FilterDataInfoServiceImplement struct {
}

/******************************************************************************
*
* @Function Name            : NewFilterDataInfoDAOImplement
*-----------------------------------------------------------------------------
*
* @Description              : 创建一个FilterDataInfoDAOImplement实例
*
* @Return Value             : FilterDataInfoDAOImplement实例
*
* @Author                   : 代码生成器创建
*
* @Date                     : 2021-06-21 09:55:32
*
******************************************************************************/
func NewFilterDataInfoServiceImplement() *FilterDataInfoServiceImplement {
	return &FilterDataInfoServiceImplement{}
}

/******************************************************************************
*
* @Reference LAPP_GAAS_GFrame_BACKEND/services/report/FilterDataInfoService.InsertOne
*
******************************************************************************/
func (impl *FilterDataInfoServiceImplement) InsertOne(entity *model.FilterDataInfo) error {

	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	dao := dal.NewFilterDataInfoDAO(session)
	record, err := dao.SelectOne()
	if err != nil {
		return err
	}
	if record != nil {
		return grmi.NewBusinessError("已经存在相同主键的记录!")
	}
	err = dao.InsertOne(entity)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
*
* @Reference LAPP_GAAS_GFrame_BACKEND/services/report/FilterDataInfoService.DeleteOne
*
******************************************************************************/
func (impl *FilterDataInfoServiceImplement) DeleteOne() error {
	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	dao := dal.NewFilterDataInfoDAO(session)
	record, err := dao.SelectOne()
	if err != nil {
		return err
	}
	if record == nil {
		return grmi.NewBusinessError("不存在指定记录!")
	}
	err = dao.DeleteOne()
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
*
* @Reference LAPP_GAAS_GFrame_BACKEND/services/report/FilterDataInfoService.SelectOne
*
******************************************************************************/
func (impl *FilterDataInfoServiceImplement) SelectOne() (*model.FilterDataInfo, error) {
	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	dao := dal.NewFilterDataInfoDAO(session)
	result, err := dao.SelectOne()
	if err != nil {
		return nil, err
	}
	return result, nil
}

/******************************************************************************
*
* @Reference LAPP_GAAS_GFrame_BACKEND/services/report/FilterDataInfoService.UpdateOne
*
******************************************************************************/
func (impl *FilterDataInfoServiceImplement) UpdateOne(entity *model.FilterDataInfo) error {
	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	dao := dal.NewFilterDataInfoDAO(session)
	record, err := dao.SelectOne()
	if err != nil {
		return err
	}
	if record == nil {
		return grmi.NewBusinessError("不存在指定记录!")
	}
	err = dao.UpdateOne(entity)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
*
* @Reference LAPP_GAAS_GFrame_BACKEND/services/report/FilterDataInfoService.Insert
*
******************************************************************************/
func (impl *FilterDataInfoServiceImplement) Insert(entities *[]model.FilterDataInfo) error {
	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	dao := dal.NewFilterDataInfoDAO(session)

	for range *entities {
		record, err := dao.SelectOne()
		if err != nil {
			return err
		}
		if record != nil {
			return grmi.NewBusinessError("已经存在相同主键的记录!")
		}
	}

	err := dao.Insert(entities)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
*
* @Reference LAPP_GAAS_GFrame_BACKEND/services/report/FilterDataInfoService.Delete
*
******************************************************************************/
func (impl *FilterDataInfoServiceImplement) Delete(entities *[]model.FilterDataInfo) error {

	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	dao := dal.NewFilterDataInfoDAO(session)

	for range *entities {
		record, err := dao.SelectOne()
		if err != nil {
			return err
		}
		if record == nil {
			return grmi.NewBusinessError("不存在指定记录!")
		}
	}

	err := dao.Delete(entities)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
*
* @Reference LAPP_GAAS_GFrame_BACKEND/services/report/FilterDataInfoService.Select
*
******************************************************************************/
func (impl *FilterDataInfoServiceImplement) Select(urlParameters map[string]string) (interface{}, error) {
	condition := DefaultConditionOfFilterDataInfoAndPaging
	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	predicates, err := condition.BuildPredicates(urlParameters)
	if err != nil {
		return nil, err
	}
	dao := dal.NewFilterDataInfoDAO(session)
	if condition.Fill(urlParameters) {
		result, err := dao.SelectAndPaging(condition.Paging, predicates, condition.OrderByFields)
		if err != nil {
			return nil, err
		}
		return result, nil
	} else {
		result, err := dao.Select(predicates, condition.OrderByFields)
		if err != nil {
			return nil, err
		}
		return result, nil
	}
}

/******************************************************************************
*
* @Reference LAPP_GAAS_GFrame_BACKEND/services/report/FilterDataInfoService.Update
*
******************************************************************************/
func (impl *FilterDataInfoServiceImplement) Update(entities *[]model.FilterDataInfo) error {
	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	dao := dal.NewFilterDataInfoDAO(session)

	for range *entities {
		record, err := dao.SelectOne()
		if err != nil {
			return err
		}
		if record == nil {
			return grmi.NewBusinessError("不存在指定记录!")
		}
	}

	err := dao.Update(entities)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
*
* @Reference LAPP_GAAS_GFrame_BACKEND/services/report/FilterDataInfoService.Select
*
******************************************************************************/
func (impl *FilterDataInfoServiceImplement) GetDisplayData() (result []model.AWMDisplay, err error) {
	result = make([]model.AWMDisplay, 0)
	slaves := db.Eloquent.Slaves()
	if len(slaves) < 3 {
		return result, errors.New("未检测到数据源数据引擎")
	}
	engine := slaves[2]
	session := engine.NewSession()
	defer session.Close()
	//now := time.Now()
	now, _ := time.ParseInLocation(grmi.DatetimeOutFormat, "2021-06-09 21:00:00", utils.TimezoneLocation)
	tempTime, err := time.ParseDuration("-1h")
	if err != nil {
		return result, err
	}
	start := now.Add(tempTime)
	startTsString := start.Format(grmi.DatetimeOutFormat)
	endTsString := now.Format(grmi.DatetimeOutFormat)
	configDao := dal.NewArcDisplayInfoDAO(session)
	dao := dal.NewFilterDataInfoDAO(session)
	configLi, err := configDao.Select(nil, nil)
	if err != nil {
		return result, err
	}
	for _, config := range configLi {
		var displayData model.AWMDisplay
		arcLi, err := dao.SelectDisplayData(config.DescCN, startTsString, endTsString)
		if err != nil {
			return result, err
		}
		var sum float64
		for _, dataInfo := range arcLi {
			targetValue := reflect.ValueOf(dataInfo).FieldByName(config.StatisticField).String()
			target, err := strconv.ParseFloat(targetValue, 64)
			if err != nil {
				return result, err
			}
			sum += target
		}
		var avg float64
		if len(arcLi) == 0 {
			avg = 0
		} else {
			avg = sum / float64(len(arcLi))
		}
		avg, err = strconv.ParseFloat(fmt.Sprintf("%.2f", avg), 64)
		if err != nil {
			return result, err
		}
		displayData.Data = arcLi
		displayData.Start = startTsString
		displayData.End = endTsString
		displayData.Title = config.Title
		displayData.Standard = config.StandardValue
		displayData.Avg = avg
		result = append(result, displayData)
	}
	return result, nil
}
